#!/usr/bin/perl
#
# Perl version of a blm2bml converter
# (c) 2003 Wolfgang Dautermann

use warnings;
use strict ;

use XML::Writer;

my ($movieline, $time, $first) ; # local variables

my ($author,$name,$description,$email,$width,$height,@moviearray) = readblm($ARGV[0]) ;
if ($width eq "")  { $width=18 ; } ; # Defaultvalue for BLMs
if ($height eq "") { $height=8 ; } ; # Defaultvalue for BLMs

my $output = new IO::File("> $ARGV[1]");

my $writer = new XML::Writer(OUTPUT => $output);
$writer->xmlDecl();
$writer->startTag("blm",
		  "width" => $width,
		  "height" => $height,
		  "bits" => "1",
		  "channels" => "1");

$writer->startTag("header");

$writer->dataElement("title",$name) ;
$writer->dataElement("description",$description) ;
$writer->dataElement("creator","blm2bml.pl") ;
$writer->dataElement("author",$author) ;

$writer->endTag("header");

# Frames
$first=1 ; # first call
foreach $movieline (@moviearray) {
    if ($movieline =~ /^@[0-9]*/) {
	if ($first==0) { $writer->endTag("frame") ; } ;
	if ($first==1) { $first=0 ; }
	$time = substr($movieline,1) ;
	$writer->startTag("frame", "duration"=>$time );
    } else {
	$writer->dataElement("row",$movieline) ;
    } ;
}

$writer->endTag("frame") ;
$writer->endTag("blm");
$writer->end();
$output->close();

# read a blm-File, strip/parse comments and empty lines
# return Moviearray, Data from comments
sub readblm {
    my ($filename) = @_ ; # parameters
    my (@moviearray,$i,$line,$author,$name,$description,$email,$width,$height) ; # local variables
    open(FILE, "< $filename") or die "Can not open file: $filename \n";
    $i = 0 ;
    $author = ""; $name="" ; $description=""; $email="";
    $width=""; $height="";
    while (<FILE>) {
	$line = $_ ;
	if ($line =~ /^#.*$/) { # Comment
	    # parse Comments in blm-Files (author, category, etc.)
	    if (substr($line,0,11) eq "# author = ") { $author=substr($line,11); }
	    if (substr($line,0,9) eq "# name = ") {$name=substr($line,9); }
	    if (substr($line,0,16) eq "# description = ") {$description=substr($line,16); }
	    if (substr($line,0,10) eq "# email = ") {$name=substr($line,10); }
	    if (substr($line,0,10) eq "# width = ") {$width=substr($line,10); }
	    if (substr($line,0,11) eq "# height = ") {$height=substr($line,11); }
	    next
	} ; # ignore/parse comments
	if ($line =~ /^$/) { next ; }; # ignore blank lines
	$moviearray[$i++] = $line ;
    }
    return ($author, $name, $description, $email, $width, $height, @moviearray) ;
}

# read a blm-File, strip comments and empty lines
# return Moviearray
sub readbml {
    my ($filename) = @_ ; # parameters
    my $xs1 = XML::Simple->new();
    my $doc = $xs1->XMLin('xml/torus.bml') ; #($filename);
    my $title = $doc->{header}->{title} ;
    my $author = $doc->{header}->{author} ;
}

=pod

=head1 NAME

blm2bml.pl - Convert blm to bml Files

=head1 SYNOPSIS

blinkserver.pl inputfile.blm outputfile.bml

=head1 DESCRIPTION

B<This program> converts a blm File to the bml Format.

=head1 AUTHOR

Wolfgang Dautermann, E<lt>dauti@oeh.tu-graz.ac.atE<gt>

=head1 SEE ALSO

http://www.blinkenlights.de/arcade/bml.en.html
http://www.blinkenlights.de/movies.en.html

=cut

__END__

 CHANGELOG:
    [2003-01-18] initial release
